<?php

namespace Drupal\yasm_charts\Controller;

use Drupal\charts\Services\ChartsSettingsServiceInterface;
use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\yasm\Controller\Users as BaseController;
use Drupal\yasm\Services\EntitiesStatisticsInterface;
use Drupal\yasm\Utility\YasmUtility;
use Drupal\yasm_charts\Utility\YasmChartsUtility;
use Drupal\yasm_charts\Utility\YasmChartsMessagesTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Appeds charts to base Users controller.
 */
class Users extends BaseController {

  use YasmChartsMessagesTrait;

  /**
   * The charts settings.
   *
   * @var \Drupal\charts\Services\ChartsSettingsServiceInterface
   */
  protected $chartSettings;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The UUID service.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidService;

  /**
   * {@inheritdoc}
   */
  public function siteContent(Request $request) {
    $build = parent::siteContent($request);

    if ($this->isChartAvailable($build, $this->chartSettings['library'])) {
      $options = $this->chartSettings;
      $options['height'] = 500;

      $chards = YasmChartsUtility::buildCharts($build['data']['#charts'], [
        'skip' => 0,
        'type' => 'column',
        'library' => $this->chartSettings['library'],
        'options' => $options,
        'id' => $this->uuidService->generate(),
      ]);
      $build = array_merge($build, YasmUtility::columns($chards, [], 1));
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(DateFormatterInterface $dateFormatter, EntityTypeManagerInterface $entityTypeManager, MessengerInterface $messenger, ModuleHandlerInterface $module_handler, EntitiesStatisticsInterface $entities_statistics, ChartsSettingsServiceInterface $chartSettings, UuidInterface $uuidService) {
    parent::__construct($dateFormatter, $entityTypeManager, $messenger, $module_handler, $entities_statistics);

    $this->chartSettings = $chartSettings->getChartsSettings();
    $this->messenger = $messenger;
    $this->uuidService = $uuidService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('date.formatter'),
      $container->get('entity_type.manager'),
      $container->get('messenger'),
      $container->get('module_handler'),
      $container->get('yasm.entities_statistics'),
      $container->get('charts.settings'),
      $container->get('uuid')
    );
  }

}
