<?php

namespace Drupal\yasm_charts\Utility;

/**
 * Yasm charts build helper class.
 */
class YasmChartsUtility {

  /**
   * Build yasm chart.
   */
  public static function buildChart($settings) {
    return [
      '#theme' => 'yasm_chart',
      '#library' => (string) $settings['library'],
      '#categories' => isset($settings['categories']) ? $settings['categories'] : [],
      '#seriesData' => $settings['seriesData'],
      '#options' => $settings['options'],
      '#id' => 'chart-' . $settings['id'],
      '#override' => isset($settings['override']) ? $settings['override'] : [],
    ];
  }

  /**
   * Build charts array from $build page array.
   */
  public static function buildCharts($charts, $settings) {
    $build = [];
    $skip = isset($settings['skip']) ? $settings['skip'] : 1;
    foreach ($charts as $chart) {
      $labels = self::getChartCategories($chart['labels'], $skip);
      $seriesData = self::getChartSeries($chart['rows'], $settings['type'], $skip);

      $build[] = self::buildChart([
        'library'    => $settings['library'],
        'categories' => $labels,
        'seriesData' => $seriesData,
        'options'    => $settings['options'],
        'id'         => $settings['id'],
      ]);
    }

    return $build;
  }

  /**
   * Get chart array series.
   */
  public static function getChartSeries($rows, $type, $skip = 1) {
    $series = [];
    $i = 0;
    foreach ($rows as $row) {
      $label = self::getChartLabel($row, $skip);
      $row = self::skipArray($row, $skip);

      $items = [];
      foreach ($row as $col) {
        $item = isset($col['data']) ? $col['data'] : $col;
        $items[] = (int) $item;
      }

      $series[] = [
        'name'  => $label,
        'color' => self::getChartColor($i),
        'type'  => $type,
        'data'  => $items,
      ];
      $i++;
    }

    return $series;
  }

  /**
   * Get chart categories for X axis.
   */
  public static function getChartCategories($labels, $skip = 1) {
    $categries = self::skipArray($labels, $skip);
    $categries = array_values($categries);

    return $categries;
  }

  /**
   * Get colors AA colors for charts.
   */
  public static function getChartColor($i) {
    $colors = [
      0 => '#205CB7',
      1 => '#815D51',
      2 => '#5EB15A',
      3 => '#9A38B2',
      4 => '#00BFD5',
      5 => '#009A8E',
      6 => '#FFC21C',
      7 => '#009CF2',
      8 => '#FF9B0F',
      9 => '#F84C3E',
      10 => '#99C455',
      11 => '#EC2C69',
    ];

    if (isset($colors[$i])) {
      $color = $colors[$i];
    }
    else {
      $color = $colors[rand(0, count($colors))];
    }

    return $color;
  }

  /**
   * Skip some elemenys of array.
   */
  private static function skipArray($array, $skip = 1) {
    if ($skip > 0) {
      for ($i = 0; $i < $skip; $i++) {
        unset($array[$i]);
      }
    }

    return $array;
  }

  /**
   * Get chart label from skiped elements.
   */
  private static function getChartLabel($row, $skip = 1) {
    $label = t('Data');
    if ($skip > 0) {
      $row = array_slice($row, 0, $skip);
      $label = [];
      foreach ($row as $col) {
        if (is_array($col) && isset($col['data'])) {
          $label[] = (string) $col['data'];
        }
        else {
          $label[] = (string) $col;
        }
      }

      $label = implode(' - ', $label);
    }

    return $label;
  }

}
